#ifndef BASIC_TIMER_H
#define BASIC_TIMER_H

#include <stm32f0xx.h>
#include <basic-timer-init.hpp>

namespace stm32f072xb {
/* Class for timer 6 and 7 (basic timers) */
class BasicTimer {
   public:
	/* Constructor with timer definition */
	explicit BasicTimer(const BasicTimerInit& init);

	/* Destructor */
	~BasicTimer();

	/** Enables counter & peripheral clock (p. 123)
	 * 	\param	[const uint16_t]			autoReloadValue Auto-reload's value
	 * 	\param	[const uint16_t]			prescalerValue Prescaler's value
	 */
	void start(const uint16_t autoReloadValue, const uint16_t prescalerValue);

	/* Disables counter */
	void stop();

	/** Getter for timer counter's value
	 * 	\return [uint16_t] 	Counter's value
	 */
	uint16_t getCounter() const;

	/** Getter for the prescaler value
	 * 	\return [uint16_t] 	Prescaler's value
	 */
	uint16_t getPrescaler() const;

	/** Getter for the auto-reload value -- p. 558
	 * 	\return [uint16_t] 	Auto-reload's value
	 */
	uint16_t getAutoReload() const;

	/** Setter for the timer's prescaler value -- p. 558
	 *  \param	[const uint16_t]	prescaler		New prescaler value
	 */
	void setPrescaler(const uint16_t prescaler);

	/** Setter for the timer's auto-reload value -- p. 558
	 *  \param	[const uint16_t]	autoReload		New auto-reload value
	 */
	void setAutoReload(const uint16_t autoReload);

   private:
	TIM_TypeDef* _timer;
	uint16_t _prescaler = 0;
	uint16_t _autoReload = 0;

	/** Setter for the timer counter's value -- p. 557
	 *  \param	[const uint16_t]	counterValue	New counter value
	 */
	void setCounter(const uint16_t counterValue);

	/** Getter for BasicTimer attribute (timer)
	 * 	\return	[TIM_TypeDef*] 	A pointer for the timer in use
	 */
	const TIM_TypeDef* getTimer() const;

	/* Enables counter */
	void enableCounter() const;

	/* Disables counter */
	void disableCounter() const;

	/* Enables uev update */
	void enableUevUpdate() const;

	/* Disables uev update */
	void disableUevUpdate() const;

	/* Enables update request source */
	void enableUpdateRequestSource() const;

	/* Disables update request source */
	void disableUpdateRequestSource() const;

	/* Enables one-pulse mode */
	void enableOnePulse() const;

	/* Disables one-pulse mode */
	void disableOnePulse() const;

	/* Enables auto-reload */
	void enableAutoReload() const;

	/* Disables auto-reload */
	void disableAutoReload() const;

	/**
	 *  Sets control register 2 for the timer in use by bits
	 * 	\param	[const MasterModeSelection&]	masterModeSelection		Master
	 * mode selection Warning: MasterModeSelection[2] is never used, hence it is
	 * never specified
	 */
	void setMasterModeSelection(const MasterModeSelection& masterModeSelection) const;

	/* Enables interruption -- p. 556 */
	void enableInterruptUpdate() const;

	/* Disables interruption */
	void disableInterruptUpdate() const;

	/* Enables timer clock */
	void enableClock() const;

	/* Enables board interruption for timer */
	void enableBoardInterruption() const;

	/* Disables board interruption for timer */
	void disableBoardInterruption() const;

	/* Sets priority of the interruption */
	void setInterruptionPriority(const uint32_t priority) const;

	/* Creates an update to the register of the prescaler and auto-reload
	 * registers with an "interruption pulse" */
	void updateRegisters();
};
}  // namespace stm32f072xb

#endif  // BASIC_TIMER_H
